@@ -1,8 +1,8 @@
 #!/bin/sh
 
 # First run alex on Lexer.x to generate Lexer.hs
-./runalex.sh
+#./runalex.sh
 
 # do a clean build of all, including the tests
 #cabal clean && cabal configure -fbuildtests && cabal build && cabal haddock
-cabal clean && cabal configure --enable-tests && cabal build && cabal test && cabal haddock
\ No newline at end of file
+cabal clean && cabal configure --enable-tests && cabal build && cabal test && cabal haddock
